if (Sys.getenv("R_OSTYPE") == "windows" || .Platform$OS.type == "windows") {
    importFrom(utils, winProgressBar, getWinProgressBar, setWinProgressBar)
}
importFrom(utils, txtProgressBar, getTxtProgressBar, setTxtProgressBar,
    flush.console)
importFrom(parallel, mclapply, parLapply, parLapplyLB)
export(pbapply, pbsapply, pblapply, pbreplicate,
    pbmapply, pb.mapply, pbtapply, pbwalk,
    pbeapply, pbvapply, pbby, pbMap,
    startpb, setpb, getpb, closepb, dopb, doshiny, pboptions, pbtypes,
    timerProgressBar, setTimerProgressBar, getTimerProgressBar,
    splitpb, getTimeAsString,
    .pb_env)
S3method("pbby", "default")
S3method("pbby", "data.frame")
